import pymunk as p
from pymunk.vec2d import Vec2d
import unittest

####################################################################

class UnitTestGeneral(unittest.TestCase):

    def testGeneral(self):
        p.version
        p.inf
        p.chipmunk_version

    def testMomentHelpers(self):
        m = p.moment_for_circle(1,2,3,(1,2))
        self.assertAlmostEqual(m, 11.5)

        m = p.moment_for_segment(1, (-10,0), (10,0), 1)
        self.assertAlmostEqual(m, 40.6666666666)

        m = p.moment_for_poly(1, [(0,0), (10,10), (10,0)], (1,2), 3)
        self.assertAlmostEqual(m, 98.3333333333)

        m = p.moment_for_box(1, (2, 3))
        self.assertAlmostEqual(m, 1.08333333333)

    def testAreaHelpers(self):
        a = p.area_for_circle(1,2)
        self.assertAlmostEqual(a, 9.4247779607)

        a = p.area_for_segment((-10,0), (10,0), 3)
        self.assertAlmostEqual(a, 148.27433388)

        a = p.area_for_poly([(0,0), (10,10), (10,0)], 3)
        self.assertAlmostEqual(a, 80.700740753)

class UnitTestBugs(unittest.TestCase):
    def testManyBoxCrash(self):
        space = p.Space()
        for x in [1,2]:
            for y in range(16):
                size = 10
                box_points = map(Vec2d, [(-size, -size), (-size, size), (size,size), (size, -size)])
                body = p.Body(10,20)
                shape = p.Poly(body, list(box_points))
                space.add(body, shape)
            space.step(1/50.0)


    def testNoStaticShape(self):
        space = p.Space()

        b1 = p.Body(1, p.inf)
        c1 = p.Circle(b1, 10)
        c1.name = "c1"
        c1.collision_type = 2

        b2 = p.Body(1, p.inf)
        c2 = p.Circle(b2, 10)
        c2.name = "c2"

        b3 = p.Body(1, p.inf)
        c3 = p.Circle(b3, 10)
        c3.name = "c3"

        b1.position = 0,0
        b2.position = 9,0
        b3.position = -9,0

        space.add(b1,c1,b2,c2,b3,c3)

        def remove_first(arbiter, space, data):
            first_shape = arbiter.shapes[0]
            #print "REMOVE FIRST", first_shape.name
            if c1 in space.shapes:
                space.remove(c1)
            #space.add_post_step_callback(space.remove, first_shape, first_shape.body)

        space.add_collision_handler(2, 0).separate = remove_first

        space.step(1./60)
        b2.position = 22,0
        space.step(1./60)





####################################################################
if __name__ == "__main__":
    print ("testing pymunk version " + p.version)
    unittest.main()



__docformat__ = "reStructuredText"

from collections import namedtuple

class PointQueryInfo(
    namedtuple('PointQueryInfo', ['shape', 'point', 'distance', 'gradient'])):
    """PointQueryInfo holds the result of a point query made on a Shape or
    Space.
    
    The properties are as follows
    
    shape 
    The nearest shape, None if no shape was within range.
    
    point
    The closest point on the shape's surface. (in world space
    coordinates)
    
    distance
    The distance to the point. The distance is negative if the
    point is inside the shape.
        
    gradient
    The gradient of the signed distance function.

        The value should be similar to
        PointQueryInfo.point/PointQueryInfo.distance, but accurate even for
        very small values of info.distance.
    """
    __slots__ = ()

class SegmentQueryInfo(
    namedtuple('SegmentQueryInfo', ['shape', 'point', 'normal', 'alpha'])):
    """Segment queries return more information than just a simple yes or no,
    they also return where a shape was hit and it's surface normal at the hit
    point. This object hold that information.

    To test if the query hit something, check if
    SegmentQueryInfo.shape == None or not.

    Segment queries are like ray casting, but because not all spatial indexes
    allow processing infinitely long ray queries it is limited to segments.
    In practice this is still very fast and you don't need to worry too much
    about the performance as long as you aren't using extremely long segments
    for your queries.

    The properties are as follows

    shape
    Shape that was hit, or None if no collision occured
    
    point 
    The point of impact.

    normal
    The normal of the surface hit.
    
    alpha
    The normalized distance along the query segment in the range [0, 1]

    """
    __slots__ = ()
    
class ShapeQueryInfo(
    namedtuple('ShapeQueryInfo', ['shape', 'contact_point_set'])):
    """Shape queries return more information than just a simple yes or no,
    they also return where a shape was hit. This object hold that information.
    """
    __slots__ = ()



# ----------------------------------------------------------------------------
# pymunk
# Copyright (c) 2007-2012 Victor Blomqvist
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# ----------------------------------------------------------------------------

"""Use this module to set runtime options of pymunk.

Currently there is one option that can be changed, debug. By setting debug to 
false debug print outs will be limited. In order to remove all debug prints 
you will also need to compile chipmunk in release mode. See 
:ref:`compile-chipmunk` for details on how to compile chipmunk.
"""

__version__ = "$Id:$"
__docformat__ = "reStructuredText"

options = {
    "debug" : True
}
"""Global dict of pymunk options.
To change make sure you import pymunk before any sub-packages and then set the 
option you want. For example::

    import pymunkoptions
    pymunkoptions.options["debug"] = False
    import pymunk
    
    #..continue to use pymunk as you normally do
    
"""



import pymunk as p
from pymunk.vec2d import Vec2d
import unittest
import pickle

####################################################################

class UnitTestShape(unittest.TestCase):
    def testPointQuery(self):
        b = p.Body(10,10)
        c = p.Circle(b, 5)
        c.cache_bb()
        
        distance, info = c.point_query((0,0))
        
        self.assertEqual(distance, -5)
        self.assertEqual(info.shape, c)
        
        self.assertEqual(info.point, (0, 0))
        self.assertEqual(info.distance, -5)
        self.assertEqual(info.gradient, (0,1))

        distance, info = c.point_query((11,0))
        
        self.assertEqual(distance, 6)
        self.assertEqual(info.shape, c)
        self.assertEqual(info.point, (5,0))
        self.assertEqual(info.distance, 6)
        self.assertEqual(info.gradient, (1,0))

    def testSegmentQuery(self):
        s = p.Space()
        b = p.Body(10,10)
        c = p.Circle(b, 5)
        c.cache_bb()

        info = c.segment_query((10,-50), (10,50))
        self.assertEqual(info.shape, None)
        self.assertEqual(info.point, (10,50))
        self.assertEqual(info.normal, (0,0))
        self.assertEqual(info.alpha, 1.0)

        info = c.segment_query((10,-50), (10,50), 6)
        self.assertEqual(info.shape, c)

        info = c.segment_query((0,-50), (0,50))
        self.assertEqual(info.shape, c)
        self.assertAlmostEqual(info.point.x, 0)
        self.assertAlmostEqual(info.point.y, -5)
        self.assertAlmostEqual(info.normal.x, 0)
        self.assertAlmostEqual(info.normal.y, -1)
        self.assertEqual(info.alpha, 0.45)

    def testMass(self):
        c = p.Circle(None, 1)
        self.assertEqual(c.mass, 0)
        c.mass = 2
        self.assertEqual(c.mass, 2)

    def testDensity(self):
        c = p.Circle(None, 1)
        self.assertEqual(c.density, 0)
        c.density = 2
        self.assertEqual(c.density, 2)

    def testMoment(self):
        c = p.Circle(None, 5)
        self.assertEqual(c.moment, 0)
        c.density = 2
        self.assertAlmostEqual(c.moment, 1963.4954084936207)
        c.density = 0
        c.mass = 2
        self.assertAlmostEqual(c.moment, 25)

    def testArea(self):
        c = p.Circle(None, 5)
        self.assertEqual(c.area, 78.53981633974483)
        
    def testCenterOfGravity(self):
        c = p.Circle(None, 5)
        self.assertEqual(c.center_of_gravity.x, 0)
        self.assertEqual(c.center_of_gravity.y, 0)
        c = p.Circle(None, 5, (10,5))
        self.assertEqual(c.center_of_gravity.x, 10)
        self.assertEqual(c.center_of_gravity.y, 5)

    def testNoBody(self):
        c = p.Circle(None, 1)
        self.assertEqual(c.body, None)

    def testRemoveBody(self):
        b = p.Body(1,1)
        c = p.Circle(b,1)
        c.body = None

        self.assertEqual(c.body, None)
        self.assertEqual(len(b.shapes), 0)

    def testSwitchBody(self):
        b1 = p.Body(1,1)
        b2 = p.Body(1,1)
        c = p.Circle(b1,1)
        self.assertEqual(c.body, b1)
        self.assertTrue(c in b1.shapes)
        self.assertTrue(c not in b2.shapes)
        c.body = b2
        self.assertEqual(c.body, b2)
        self.assertTrue(c not in b1.shapes)
        self.assertTrue(c in b2.shapes)

    def testSensor(self):
        b1 = p.Body(1,1)
        c = p.Circle(b1,1)
        self.assertFalse(c.sensor)
        c.sensor = True
        self.assertTrue(c.sensor)

    def testElasticity(self):
        b1 = p.Body(1,1)
        c = p.Circle(b1,1)
        self.assertEqual(c.elasticity, 0)
        c.elasticity = 1
        self.assertEqual(c.elasticity, 1)

    def testFriction(self):
        b1 = p.Body(1,1)
        c = p.Circle(b1,1)
        self.assertEqual(c.friction, 0)
        c.friction = 1
        self.assertEqual(c.friction, 1)

    def testSurfaceVelocity(self):
        b1 = p.Body(1,1)
        c = p.Circle(b1,1)
        self.assertEqual(c.surface_velocity, (0,0))
        c.surface_velocity = (1,2)
        self.assertEqual(c.surface_velocity, (1,2))

    def testCollisionType(self):
        b1 = p.Body(1,1)
        c = p.Circle(b1,1)
        self.assertEqual(c.collision_type, 0)
        c.collision_type = 1
        self.assertEqual(c.collision_type, 1)

    def testFilter(self):
        b1 = p.Body(1,1)
        c = p.Circle(b1,1)
        self.assertEqual(c.filter, p.ShapeFilter(0, 0xffffffff, 0xffffffff))
        c.filter = p.ShapeFilter(1, 0xfffffff2, 0xfffffff3)
        self.assertEqual(c.filter, p.ShapeFilter(1, 0xfffffff2, 0xfffffff3))

    def testSpace(self):
        b1 = p.Body(1,1)
        c = p.Circle(b1,1)
        self.assertEqual(c.space, None)
        s = p.Space()
        s.add(c)
        self.assertEqual(c.space, s)
        
    def testShapesCollide(self):
        b1 = p.Body(1,1)
        s1 = p.Circle(b1, 10)
        
        b2 = p.Body(1,1)
        b2.position = 30,30
        s2 = p.Circle(b2, 10)
        
        c = s1.shapes_collide(s2)
        self.assertEqual(c.normal, (1, 0))
        self.assertEqual(len(c.points), 1)
        point = c.points[0]
        self.assertEqual(point.point_a, (10,0))
        self.assertEqual(point.point_b, (-10,0))
        self.assertEqual(point.distance, -20) 

    def testPickle(self):
        b = p.Body(1,2)
        c = p.Circle(b, 3, (4,5))
        c.sensor = True
        c.collision_type = 6
        c.filter = p.ShapeFilter()
        c.elasticity = 7
        c.friction = 8
        c.surface_velocity = (9,10)

        s = pickle.dumps(c)
        c2 = pickle.loads(s)

        self.assertEqual(c.sensor, c2.sensor)
        self.assertEqual(c.collision_type, c2.collision_type)
        self.assertEqual(c.filter, c2.filter)
        self.assertEqual(c.elasticity, c2.elasticity)
        self.assertEqual(c.friction, c2.friction)
        self.assertEqual(c.surface_velocity, c2.surface_velocity)
        self.assertEqual(c.density, c2.density)
        self.assertEqual(c.mass, c2.mass)
        self.assertEqual(c.body.mass, c2.body.mass)

        c = p.Circle(None,1)
        c.density = 3

        s = pickle.dumps(c)
        c2 = pickle.loads(s)

        self.assertEqual(c.mass, c2.mass)
        self.assertEqual(c.density, c2.density)

        c2 = c.copy()
        


class UnitTestCircle(unittest.TestCase):
    def testCircleBB(self):
        s = p.Space()
        b = p.Body(10,10)
        c = p.Circle(b,5)

        c.cache_bb()

        self.assertEqual(c.bb, p.BB(-5.0, -5.0, 5.0, 5.0))

    def testCircleNoBody(self):
        s = p.Space()
        c = p.Circle(None,5)

        bb = c.update(p.Transform(1, 2, 3, 4, 5, 6))
        self.assertEqual(c.bb, bb)
        self.assertEqual(c.bb, p.BB(0, 1, 10, 11))

    def testOffset(self):
        c = p.Circle(None, 5, (1,2))

        self.assertEqual(c.offset, (1,2))

    def testOffsetUnsafe(self):
        c = p.Circle(None, 5, (1,2))

        c.unsafe_set_offset((3,4))

        self.assertEqual(c.offset, (3,4))

    def testRadius(self):
        c = p.Circle(None, 5)

        self.assertEqual(c.radius, 5)

    def testRadiusUnsafe(self):
        c = p.Circle(None, 5)
        
        c.unsafe_set_radius(3)
        
        self.assertEqual(c.radius, 3)

    def testPickle(self):
        c = p.Circle(None, 3, (4,5))

        s = pickle.dumps(c)
        c2 = pickle.loads(s)

        self.assertEqual(c.radius, c2.radius)
        self.assertEqual(c.offset, c2.offset)

class UnitTestSegment(unittest.TestCase):
    def testBB(self):
        s = p.Space()
        b = p.Body(10,10)
        c = p.Segment(b,(2,2),(2,3),2)

        c.cache_bb()

        self.assertEqual(c.bb, p.BB(0, 0, 4.0, 5.0))

    def testProperties(self):
        c = p.Segment(None, (2,2), (2,3), 4)

        self.assertEqual(c.a, (2,2))
        self.assertEqual(c.b, (2,3))
        self.assertEqual(c.normal, (1,0))
        self.assertEqual(c.radius, 4)

    def testPropertiesUnsafe(self):
        c = p.Segment(None, (2,2), (2,3), 4)
        
        c.unsafe_set_endpoints((3,4), (5,6))
        self.assertEqual(c.a, (3,4))
        self.assertEqual(c.b, (5,6))
        
        c.unsafe_set_radius(5)
        self.assertEqual(c.radius, 5)

    def testSetNeighbors(self):
        c = p.Segment(None, (2,2), (2,3), 1)
        c.set_neighbors((2,2),(2,3))

    def testSegmentSegmentCollision(self):
        s = p.Space()
        b1 = p.Body(10,10)
        c1 = p.Segment(b1, (-1,-1), (1,1), 1)
        b2 = p.Body(10,10)
        c2 = p.Segment(b2, (1,-1), (-1,1), 1)

        s.add(b1,b2,c1,c2)

        self.num_of_begins = 0
        def begin(arb, space, data):
            self.num_of_begins += 1
            return True
            
        s.add_default_collision_handler().begin=begin
        s.step(.1)

        self.assertEqual(1, self.num_of_begins)

    def testPickle(self):
        c = p.Segment(None, (1,2), (3,4), 5)

        s = pickle.dumps(c)
        c2 = pickle.loads(s)

        self.assertEqual(c.a, c2.a)
        self.assertEqual(c.b, c2.b)
        self.assertEqual(c.radius, c2.radius)

class UnitTestPoly(unittest.TestCase):
    def testInit(self):
        c = p.Poly(None, [(0,0),(10,10),(20,0),(-10,10)], None, 0)

        b = p.Body(1,2)
        c = p.Poly(b, [(0,0),(10,10),(20,0),(-10,10)], p.Transform.identity(), 6)

    def testVertices(self):
        vs = [(-10,10), (0,0),(20,0),(10,10)]
        c = p.Poly(None, vs, None, 0)

        self.assertEqual(c.get_vertices(), vs)
        
        c = p.Poly(None, vs, p.Transform(1,2,3,4,5,6), 0)
        
        vs2 = [(5.0, 6.0), (25.0, 26.0), (45.0, 66.0), (25.0, 46.0)]
        self.assertEqual(c.get_vertices(), vs2)

    def testVerticesUnsafe(self):
        vs = [(-10,10), (0,0),(20,0),(10,10)]
        c = p.Poly(None, vs, None, 0)

        vs2 = [(-3,3), (0,0), (3,0)]
        c.unsafe_set_vertices(vs2)
        self.assertEqual(c.get_vertices(), vs2)
        
        vs3 = [(-4,4), (0,0), (4,0)]
        c.unsafe_set_vertices(vs3, p.Transform.identity())
        self.assertEqual(c.get_vertices(), vs3)
        
    def testBB(self):
        c = p.Poly(None, [(2,2),(4,3),(3,5)])
        bb = c.update(p.Transform.identity())
        self.assertEqual(bb, c.bb)
        self.assertEqual(c.bb, p.BB(2, 2, 4, 5))

        b = p.Body(1,2)
        c = p.Poly(b, [(2,2),(4,3),(3,5)])
        c.cache_bb()
        self.assertEqual(c.bb, p.BB(2, 2, 4, 5))

        s = p.Space()
        b = p.Body(1,2)
        c = p.Poly(b, [(2,2),(4,3),(3,5)])
        s.add(b,c)
        self.assertEqual(c.bb, p.BB(2, 2, 4, 5))

    def testRadius(self):
        c = p.Poly(None, [(2,2), (4,3), (3,5)], radius=10)
        self.assertEqual(c.radius, 10)
        
    def testRadiusUnsafe(self):
        c = p.Poly(None, [(2,2), (4,3), (3,5)], radius=10)

        c.unsafe_set_radius(20)

        self.assertEqual(c.radius, 20)        

    def testCreateBox(self):
        c = p.Poly.create_box(None, (4,2), 3)
        self.assertEqual(c.get_vertices(), [(2,-1), (2,1), (-2,1), (-2,-1)])

        c = p.Poly.create_box_bb(None, p.BB(1,2,3,4), 3)
        self.assertEqual(c.get_vertices(), [(3,2), (3,4), (1,4), (1,2)])


    def testPickle(self):
        c = p.Poly(None, [(1,2), (3,4), (5,6)], radius = 5)

        s = pickle.dumps(c)
        c2 = pickle.loads(s)

        self.assertEqual(c.get_vertices(), c2.get_vertices())
        self.assertEqual(c.radius, c2.radius)

####################################################################
if __name__ == "__main__":
    print ("testing pymunk version " + p.version)
    unittest.main()



__docformat__ = "reStructuredText"

from collections import namedtuple

from ._chipmunk_cffi import lib, ffi 

from .vec2d import Vec2d
from .body import Body

class SpaceDebugColor(namedtuple("SpaceDebugColor", ["r","g","b","a"])):
    """Color tuple used by the debug drawing API.
    """
    __slots__ = ()    

    def as_int(self):
        return int(self[0]), int(self[1]), int(self[2]), int(self[3])

    def as_float(self):
        return self[0]/255., self[1]/255., self[2]/255., self[3]/255.


class SpaceDebugDrawOptions(object):
    """SpaceDebugDrawOptions configures debug drawing. 

    If appropriate its usually easy to use the supplied draw implementations 
    directly: pymunk.pygame_util, pymunk.pyglet_util and pymunk.matplotlib_util.
    """

    DRAW_SHAPES = lib.CP_SPACE_DEBUG_DRAW_SHAPES
    """Draw shapes.  
    
    Use on the flags property to control if shapes should be drawn or not.
    """

    DRAW_CONSTRAINTS = lib.CP_SPACE_DEBUG_DRAW_CONSTRAINTS
    """Draw constraints. 
    
    Use on the flags property to control if constraints should be drawn or not.
    """

    DRAW_COLLISION_POINTS = lib.CP_SPACE_DEBUG_DRAW_COLLISION_POINTS
    """Draw collision points.
    
    Use on the flags property to control if collision points should be drawn or
    not.
    """

    shape_dynamic_color = SpaceDebugColor(52,152,219,255)
    shape_static_color = SpaceDebugColor(149,165,166,255)
    shape_kinematic_color = SpaceDebugColor(39,174,96,255)
    shape_sleeping_color = SpaceDebugColor(114,148,168,255)

    def __init__(self):
        _options = ffi.new("cpSpaceDebugDrawOptions *")
        self._options = _options        
        self.shape_outline_color = (44,62,80,255)
        self.constraint_color = (142,68,173,255)
        self.collision_point_color = (231,76,60,255)  
        
        # Set to false to bypass chipmunk shape drawing code
        self._use_chipmunk_debug_draw = True 
        
        
        @ffi.callback("cpSpaceDebugDrawCircleImpl")
        def f1(pos, angle, radius, outline_color, fill_color, data):
            self.draw_circle(
                Vec2d._fromcffi(pos), angle, radius, 
                self._c(outline_color), self._c(fill_color))
        _options.drawCircle = f1

        @ffi.callback("cpSpaceDebugDrawSegmentImpl")
        def f2(a, b, color, data):
            self.draw_segment(
                Vec2d._fromcffi(a), Vec2d._fromcffi(b), self._c(color))
        _options.drawSegment = f2

        @ffi.callback("cpSpaceDebugDrawFatSegmentImpl")
        def f3(a, b, radius, outline_color, fill_color, data):
            self.draw_fat_segment(
                Vec2d._fromcffi(a), Vec2d._fromcffi(b), radius, 
                self._c(outline_color), self._c(fill_color))
        _options.drawFatSegment = f3

        @ffi.callback("cpSpaceDebugDrawPolygonImpl")
        def f4(count, verts, radius, outline_color, fill_color, data):
            vs = []
            for i in range(count):
                vs.append(Vec2d._fromcffi(verts[i]))
            self.draw_polygon(
                vs, radius, 
                self._c(outline_color), self._c(fill_color))   
        _options.drawPolygon = f4

        @ffi.callback("cpSpaceDebugDrawDotImpl")
        def f5(size, pos, color, data):
            self.draw_dot(size, Vec2d._fromcffi(pos), self._c(color))
        _options.drawDot = f5

        @ffi.callback("cpSpaceDebugDrawColorForShapeImpl")
        def f6(_shape, data):
            space = ffi.from_handle(data)
            shape = space._get_shape(_shape)
            return self.color_for_shape(shape)
        _options.colorForShape = f6

        self.flags = SpaceDebugDrawOptions.DRAW_SHAPES | \
                SpaceDebugDrawOptions.DRAW_CONSTRAINTS | \
                SpaceDebugDrawOptions.DRAW_COLLISION_POINTS

        self._callbacks = [f1,f2,f3,f4,f5,f6]

    def _get_shape_outline_color(self):
        return self._c(self._options.shapeOutlineColor)
    def _set_shape_outline_color(self, c):
        self._options.shapeOutlineColor = c
    shape_outline_color = property(_get_shape_outline_color, 
        _set_shape_outline_color,
        doc="""The outline color of shapes.
        
        Should be a tuple of 4 ints between 0 and 255 (r,g,b,a).

        Example:

        >>> import pymunk
        >>> s = pymunk.Space()
        >>> c = pymunk.Circle(s.static_body, 10)
        >>> s.add(c)
        >>> options = pymunk.SpaceDebugDrawOptions()
        >>> s.debug_draw(options)
        draw_circle (Vec2d(0.0, 0.0), 0.0, 10.0, SpaceDebugColor(r=44.0, g=62.0, b=80.0, a=255.0), SpaceDebugColor(r=149.0, g=165.0, b=166.0, a=255.0))
        >>> options.shape_outline_color = (10,20,30,40)
        >>> s.debug_draw(options)
        draw_circle (Vec2d(0.0, 0.0), 0.0, 10.0, SpaceDebugColor(r=10.0, g=20.0, b=30.0, a=40.0), SpaceDebugColor(r=149.0, g=165.0, b=166.0, a=255.0))

        """)

    def _get_constraint_color(self):
        return self._c(self._options.constraintColor)
    def _set_constraint_color(self, c):
        self._options.constraintColor = c
    constraint_color = property(_get_constraint_color, 
        _set_constraint_color,
        doc="""The color of constraints.

        Should be a tuple of 4 ints between 0 and 255 (r,g,b,a).
        
        Example:

        >>> import pymunk
        >>> s = pymunk.Space()
        >>> j = pymunk.PivotJoint(s.static_body, s.static_body, (0,0))
        >>> s.add(j)
        >>> options = pymunk.SpaceDebugDrawOptions()
        >>> s.debug_draw(options)
        draw_dot (5.0, Vec2d(0.0, 0.0), SpaceDebugColor(r=142.0, g=68.0, b=173.0, a=255.0))
        draw_dot (5.0, Vec2d(0.0, 0.0), SpaceDebugColor(r=142.0, g=68.0, b=173.0, a=255.0))
        >>> options.constraint_color = (10,20,30,40)
        >>> s.debug_draw(options)
        draw_dot (5.0, Vec2d(0.0, 0.0), SpaceDebugColor(r=10.0, g=20.0, b=30.0, a=40.0))
        draw_dot (5.0, Vec2d(0.0, 0.0), SpaceDebugColor(r=10.0, g=20.0, b=30.0, a=40.0))

        """)

    def _get_collision_point_color(self):
        return self._c(self._options.collisionPointColor)
    def _set_collision_point_color(self, c):
        self._options.collisionPointColor = c
    collision_point_color = property(_get_collision_point_color, 
        _set_collision_point_color,
        doc="""The color of collisions.

        Should be a tuple of 4 ints between 0 and 255 (r,g,b,a).
        
        Example:

        >>> import pymunk
        >>> s = pymunk.Space()
        >>> b = pymunk.Body(1,10)
        >>> c1 = pymunk.Circle(b, 10)
        >>> c2 = pymunk.Circle(s.static_body, 10)
        >>> s.add(b, c1, c2)
        >>> s.step(1)
        >>> options = pymunk.SpaceDebugDrawOptions()
        >>> s.debug_draw(options)
        draw_circle (Vec2d(0.0, 0.0), 0.0, 10.0, SpaceDebugColor(r=44.0, g=62.0, b=80.0, a=255.0), SpaceDebugColor(r=52.0, g=152.0, b=219.0, a=255.0))
        draw_circle (Vec2d(0.0, 0.0), 0.0, 10.0, SpaceDebugColor(r=44.0, g=62.0, b=80.0, a=255.0), SpaceDebugColor(r=149.0, g=165.0, b=166.0, a=255.0))
        draw_segment (Vec2d(8.0, 0.0), Vec2d(-8.0, 0.0), SpaceDebugColor(r=231.0, g=76.0, b=60.0, a=255.0))
        >>> options.collision_point_color = (10,20,30,40)
        >>> s.debug_draw(options)
        draw_circle (Vec2d(0.0, 0.0), 0.0, 10.0, SpaceDebugColor(r=44.0, g=62.0, b=80.0, a=255.0), SpaceDebugColor(r=52.0, g=152.0, b=219.0, a=255.0))
        draw_circle (Vec2d(0.0, 0.0), 0.0, 10.0, SpaceDebugColor(r=44.0, g=62.0, b=80.0, a=255.0), SpaceDebugColor(r=149.0, g=165.0, b=166.0, a=255.0))
        draw_segment (Vec2d(8.0, 0.0), Vec2d(-8.0, 0.0), SpaceDebugColor(r=10.0, g=20.0, b=30.0, a=40.0))
        """)

    def __enter__(self):
        pass
    def __exit__(self, type, value, traceback):
        pass

    def _c(self, color):
        return SpaceDebugColor(color.r, color.g, color.b, color.a)

    def _get_flags(self):
        return self._options.flags
    def _set_flags(self, f):
        self._options.flags = f
    flags = property(_get_flags, _set_flags,
        doc="""Bit flags which of shapes, joints and collisions should be drawn.

        By default all 3 flags are set, meaning shapes, joints and collisions 
        will be drawn.

        Example using the basic text only DebugDraw implementation (normally
        you would the desired backend instead, such as 
        `pygame_util.DrawOptions` or `pyglet_util.DrawOptions`):

        >>> import pymunk
        >>> s = pymunk.Space()
        >>> b = pymunk.Body()
        >>> c = pymunk.Circle(b, 10)
        >>> c.mass = 3
        >>> s.add(b, c)
        >>> s.add(pymunk.Circle(s.static_body, 3))
        >>> s.step(0.01)
        >>> options = pymunk.SpaceDebugDrawOptions() 
        
        >>> # Only draw the shapes, nothing else:
        >>> options.flags = pymunk.SpaceDebugDrawOptions.DRAW_SHAPES
        >>> s.debug_draw(options) 
        draw_circle (Vec2d(0.0, 0.0), 0.0, 10.0, SpaceDebugColor(r=44.0, g=62.0, b=80.0, a=255.0), SpaceDebugColor(r=52.0, g=152.0, b=219.0, a=255.0))
        draw_circle (Vec2d(0.0, 0.0), 0.0, 3.0, SpaceDebugColor(r=44.0, g=62.0, b=80.0, a=255.0), SpaceDebugColor(r=149.0, g=165.0, b=166.0, a=255.0))

        >>> # Draw the shapes and collision points:
        >>> options.flags = pymunk.SpaceDebugDrawOptions.DRAW_SHAPES
        >>> options.flags |= pymunk.SpaceDebugDrawOptions.DRAW_COLLISION_POINTS
        >>> s.debug_draw(options)
        draw_circle (Vec2d(0.0, 0.0), 0.0, 10.0, SpaceDebugColor(r=44.0, g=62.0, b=80.0, a=255.0), SpaceDebugColor(r=52.0, g=152.0, b=219.0, a=255.0))
        draw_circle (Vec2d(0.0, 0.0), 0.0, 3.0, SpaceDebugColor(r=44.0, g=62.0, b=80.0, a=255.0), SpaceDebugColor(r=149.0, g=165.0, b=166.0, a=255.0))
        draw_segment (Vec2d(1.0, 0.0), Vec2d(-8.0, 0.0), SpaceDebugColor(r=231.0, g=76.0, b=60.0, a=255.0))
        
        """)

    def draw_circle(self, *args):
        print("draw_circle", args)

    def draw_segment(self, *args):
        print("draw_segment", args)

    def draw_fat_segment(self, *args):
        print("draw_fat_segment", args)

    def draw_polygon(self, *args):
        print("draw_polygon", args)

    def draw_dot(self, *args):
        print("draw_dot", args)

    def draw_shape(self, shape):
        print("draw_shape", shape)

    def color_for_shape(self, shape):
        if hasattr(shape, "color"):
            return shape.color

        color = self.shape_dynamic_color
        if shape.body != None:
            if shape.body.body_type == Body.STATIC:
                color = self.shape_static_color
            elif shape.body.body_type == Body.KINEMATIC:
                color = self.shape_kinematic_color
            elif shape.body.is_sleeping:
                color = self.shape_sleeping_color
                
        return color



import pymunk as p
from pymunk.vec2d import Vec2d
import unittest
import pickle

####################################################################
class UnitTestConstraint(unittest.TestCase):
    def testA(self):
        a,b = p.Body(10,10), p.Body(10,10)
        j = p.PivotJoint(a, b, (0,0))
        self.assertEqual(j.a, a)

    def testB(self):
        a,b = p.Body(10,10), p.Body(10,10)
        j = p.PivotJoint(a, b, (0,0))
        self.assertEqual(j.b, b)

    def testMaxForce(self):
        a,b = p.Body(10,10), p.Body(10,10)
        j = p.PivotJoint(a, b, (0,0))
        self.assertEqual(j.max_force, p.inf)
        j.max_force = 10
        self.assertEqual(j.max_force, 10)

    def testErrorBias(self):
        a,b = p.Body(10,10), p.Body(10,10)
        j = p.PivotJoint(a, b, (0,0))
        self.assertAlmostEqual(j.error_bias, pow(1.0 - 0.1, 60.0))
        j.error_bias = 0.3
        self.assertEqual(j.error_bias, 0.3)

    def testMaxBias(self):
        a,b = p.Body(10,10), p.Body(10,10)
        j = p.PivotJoint(a, b, (0,0))
        self.assertEqual(j.max_bias, p.inf)
        j.max_bias = 10
        self.assertEqual(j.max_bias, 10)

    def testCollideBodies(self):
        a,b = p.Body(10,10), p.Body(10,10)
        j = p.PivotJoint(a, b, (0,0))
        self.assertEqual(j.collide_bodies, True)
        j.collide_bodies = False
        self.assertEqual(j.collide_bodies, False)

    def testImpulse(self):
        a,b = p.Body(10,10), p.Body(10,10)
        b.position = 0,10
        j = p.PivotJoint(a, b, (0,0))

        s = p.Space()
        s.gravity = 0,10
        s.add(b, j)
        self.assertEqual(j.impulse, 0)
        s.step(1)
        self.assertAlmostEqual(j.impulse, 50)

    def testActivate(self):
        a,b = p.Body(4,5), p.Body(10,10)
        j = p.PivotJoint(a,b,(0,0))
        s = p.Space()
        s.sleep_time_threshold = 0.01
        s.add(a,b)
        a.sleep()
        b.sleep()

        j.activate_bodies()
        self.assertFalse(a.is_sleeping)
        self.assertFalse(b.is_sleeping)

    def testPickle(self):
        a,b = p.Body(4,5), p.Body(10,10)
        a.custom = "a"
        b.custom = "b"
        j = p.PivotJoint(a,b,(1,2))
        j.custom = "test"
        j.max_force = 2
        j.error_bias = 3
        j.max_bias = 4
        j.collide_bodies = False

        s = pickle.dumps(j)
        j2 = pickle.loads(s)
        self.assertEqual(j.custom, j2.custom)
        self.assertEqual(j.max_force, j2.max_force)
        self.assertEqual(j.error_bias, j2.error_bias)
        self.assertEqual(j.max_bias, j2.max_bias)
        self.assertEqual(j.collide_bodies, j2.collide_bodies)
        self.assertEqual(j.a.custom, j2.a.custom)
        self.assertEqual(j.b.custom, j2.b.custom)

        j2 = j.copy()

class UnitTestPinJoint(unittest.TestCase):
    def testAnchor(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.PinJoint(a, b, (1,2), (3,4))
        self.assertEqual(j.anchor_a, (1,2))
        self.assertEqual(j.anchor_b, (3,4))
        j.anchor_a = (5,6)
        j.anchor_b = (7,8)
        self.assertEqual(j.anchor_a, (5,6))
        self.assertEqual(j.anchor_b, (7,8))

    def testDistane(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.PinJoint(a, b, (0,0), (10,0))
        self.assertEqual(j.distance, 10)
        j.distance = 20
        self.assertEqual(j.distance, 20)

    def testPickle(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.PinJoint(a, b, (1,2), (3,4))

        s = pickle.dumps(j)
        j2 = pickle.loads(s)

        self.assertEqual(j.anchor_a, j2.anchor_a)
        self.assertEqual(j.anchor_b, j2.anchor_b)
        self.assertEqual(j.distance, j2.distance)
        self.assertEqual(j.a.mass, j2.a.mass)
        self.assertEqual(j.b.mass, j2.b.mass)

class UnitTestSlideJoint(unittest.TestCase):
    def testAnchor(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.SlideJoint(a, b, (1,2), (3,4), 0, 10)
        self.assertEqual(j.anchor_a, (1,2))
        self.assertEqual(j.anchor_b, (3,4))
        j.anchor_a = (5,6)
        j.anchor_b = (7,8)
        self.assertEqual(j.anchor_a, (5,6))
        self.assertEqual(j.anchor_b, (7,8))

    def testMin(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.SlideJoint(a, b, (0,0), (0,0), 1, 0)
        self.assertEqual(j.min, 1)
        j.min = 2
        self.assertEqual(j.min, 2)

    def testMax(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.SlideJoint(a, b, (0,0), (0,0), 0, 1)
        self.assertEqual(j.max, 1)
        j.max = 2
        self.assertEqual(j.max, 2)

    def testPickle(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.SlideJoint(a, b, (1,2), (3,4), 5, 6)

        s = pickle.dumps(j)
        j2 = pickle.loads(s)

        self.assertEqual(j.anchor_a, j2.anchor_a)
        self.assertEqual(j.anchor_b, j2.anchor_b)
        self.assertEqual(j.min, j2.min)
        self.assertEqual(j.max, j2.max)
        self.assertEqual(j.a.mass, j2.a.mass)
        self.assertEqual(j.b.mass, j2.b.mass)

class UnitTestPivotJoint(unittest.TestCase):
    def testAnchorByPivot(self):
        a,b = p.Body(10,10), p.Body(20,20)
        a.position = (5,7)
        j = p.PivotJoint(a, b, (1,2))
        self.assertEqual(j.anchor_a, (-4,-5))
        self.assertEqual(j.anchor_b, (1,2))

    def testAnchorByAnchor(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.PivotJoint(a, b, (1,2), (3,4))
        self.assertEqual(j.anchor_a, (1,2))
        self.assertEqual(j.anchor_b, (3,4))
        j.anchor_a = (5,6)
        j.anchor_b = (7,8)
        self.assertEqual(j.anchor_a, (5,6))
        self.assertEqual(j.anchor_b, (7,8))

    def testPickle(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.PivotJoint(a, b, (1,2), (3,4))

        s = pickle.dumps(j)
        j2 = pickle.loads(s)

        self.assertEqual(j.anchor_a, j2.anchor_a)
        self.assertEqual(j.anchor_b, j2.anchor_b)
        self.assertEqual(j.a.mass, j2.a.mass)
        self.assertEqual(j.b.mass, j2.b.mass)

class UnitTestGrooveJoint(unittest.TestCase):
    def testAnchor(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.GrooveJoint(a, b, (0,0), (0,0), (1,2))
        self.assertEqual(j.anchor_b, (1,2))
        j.anchor_b = (3,4)
        self.assertEqual(j.anchor_b, (3,4))

    def testGroove(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.GrooveJoint(a, b, (1,2), (3,4), (0,0))
        self.assertEqual(j.groove_a, (1,2))
        self.assertEqual(j.groove_b, (3,4))
        j.groove_a = (5,6)
        j.groove_b = (7,8)
        self.assertEqual(j.groove_a, (5,6))
        self.assertEqual(j.groove_b, (7,8))

    def testPickle(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.GrooveJoint(a, b, (1,2), (3,4), (5,6))

        s = pickle.dumps(j)
        j2 = pickle.loads(s)

        self.assertEqual(j.groove_a, j2.groove_a)
        self.assertEqual(j.groove_b, j2.groove_b)
        self.assertEqual(j.anchor_b, j2.anchor_b)
        self.assertEqual(j.a.mass, j2.a.mass)
        self.assertEqual(j.b.mass, j2.b.mass)

class UnitTestDampedSpring(unittest.TestCase):
    def testAnchor(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.DampedSpring(a, b, (1,2), (3,4), 0, 0, 0)
        self.assertEqual(j.anchor_a, (1,2))
        self.assertEqual(j.anchor_b, (3,4))
        j.anchor_a = (5,6)
        j.anchor_b = (7,8)
        self.assertEqual(j.anchor_a, (5,6))
        self.assertEqual(j.anchor_b, (7,8))

    def testRestLength(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.DampedSpring(a, b, (0,0), (0,0), 1, 0, 0)
        self.assertEqual(j.rest_length, 1)
        j.rest_length = 2
        self.assertEqual(j.rest_length, 2)

    def testStiffness(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.DampedSpring(a, b, (0,0), (0,0), 0, 1, 0)
        self.assertEqual(j.stiffness, 1)
        j.stiffness = 2
        self.assertEqual(j.stiffness, 2)

    def testDamping(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.DampedSpring(a, b, (0,0), (0,0), 0, 0, 1)
        self.assertEqual(j.damping, 1)
        j.damping = 2
        self.assertEqual(j.damping, 2)

    def testPickle(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.DampedSpring(a, b, (1,2), (3,4), 5, 6, 7)

        s = pickle.dumps(j)
        j2 = pickle.loads(s)

        self.assertEqual(j.anchor_a, j2.anchor_a)
        self.assertEqual(j.anchor_b, j2.anchor_b)
        self.assertEqual(j.rest_length, j2.rest_length)
        self.assertEqual(j.stiffness, j2.stiffness)
        self.assertEqual(j.damping, j2.damping)
        self.assertEqual(j.a.mass, j2.a.mass)
        self.assertEqual(j.b.mass, j2.b.mass)

class UnitTestDampedRotarySpring(unittest.TestCase):
    def testRestAngle(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.DampedRotarySpring(a, b, 1, 0, 0)
        self.assertEqual(j.rest_angle, 1)
        j.rest_angle = 2
        self.assertEqual(j.rest_angle, 2)

    def testStiffness(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.DampedRotarySpring(a, b, 0, 1, 0)
        self.assertEqual(j.stiffness, 1)
        j.stiffness = 2
        self.assertEqual(j.stiffness, 2)

    def testDamping(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.DampedRotarySpring(a, b,  0, 0, 1)
        self.assertEqual(j.damping, 1)
        j.damping = 2
        self.assertEqual(j.damping, 2)

    def testPickle(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.DampedRotarySpring(a, b, 1, 2, 3)

        s = pickle.dumps(j)
        j2 = pickle.loads(s)

        self.assertEqual(j.rest_angle, j2.rest_angle)
        self.assertEqual(j.stiffness, j2.stiffness)
        self.assertEqual(j.damping, j2.damping)
        self.assertEqual(j.a.mass, j2.a.mass)
        self.assertEqual(j.b.mass, j2.b.mass)

class UnitTestRotaryLimitJoint(unittest.TestCase):
    def testMin(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.RotaryLimitJoint(a, b, 1, 0)
        self.assertEqual(j.min, 1)
        j.min = 2
        self.assertEqual(j.min, 2)

    def testMax(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.RotaryLimitJoint(a, b, 0, 1)
        self.assertEqual(j.max, 1)
        j.max = 2
        self.assertEqual(j.max, 2)

    def testPickle(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.RotaryLimitJoint(a, b, 1, 2)

        s = pickle.dumps(j)
        j2 = pickle.loads(s)

        self.assertEqual(j.min, j2.min)
        self.assertEqual(j.max, j2.max)
        self.assertEqual(j.a.mass, j2.a.mass)
        self.assertEqual(j.b.mass, j2.b.mass)

class UnitTestRatchetJoint(unittest.TestCase):
    def testAngle(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.RatchetJoint(a, b, 0, 0)
        self.assertEqual(j.angle, 0)
        j.angle = 1
        self.assertEqual(j.angle, 1)

    def testPhase(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.RatchetJoint(a, b, 1, 0)
        self.assertEqual(j.phase, 1)
        j.phase = 2
        self.assertEqual(j.phase, 2)

    def testRatchet(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.RatchetJoint(a, b, 0, 1)
        self.assertEqual(j.ratchet, 1)
        j.ratchet = 2
        self.assertEqual(j.ratchet, 2)

    def testPickle(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.RatchetJoint(a, b, 1, 2)

        s = pickle.dumps(j)
        j2 = pickle.loads(s)

        self.assertEqual(j.phase, j2.phase)
        self.assertEqual(j.ratchet, j2.ratchet)
        self.assertEqual(j.a.mass, j2.a.mass)
        self.assertEqual(j.b.mass, j2.b.mass)

class UnitTestGearJoint(unittest.TestCase):
    def testPhase(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.GearJoint(a, b, 1, 0)
        self.assertEqual(j.phase, 1)
        j.phase = 2
        self.assertEqual(j.phase, 2)

    def testRatio(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.GearJoint(a, b, 0, 1)
        self.assertEqual(j.ratio, 1)
        j.ratio = 2
        self.assertEqual(j.ratio, 2)

    def testPickle(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.GearJoint(a, b, 1, 2)

        s = pickle.dumps(j)
        j2 = pickle.loads(s)

        self.assertEqual(j.phase, j2.phase)
        self.assertEqual(j.ratio, j2.ratio)
        self.assertEqual(j.a.mass, j2.a.mass)
        self.assertEqual(j.b.mass, j2.b.mass)

class UnitTestSimleMotor(unittest.TestCase):
    def testSimpleMotor(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.SimpleMotor(a, b, 0.3)
        self.assertEqual(j.rate, 0.3)
        j.rate = 0.4
        self.assertEqual(j.rate, 0.4)

    def testPickle(self):
        a,b = p.Body(10,10), p.Body(20,20)
        j = p.SimpleMotor(a, b, 1)

        s = pickle.dumps(j)
        j2 = pickle.loads(s)

        self.assertEqual(j.rate, j2.rate)
        self.assertEqual(j.a.mass, j2.a.mass)
        self.assertEqual(j.b.mass, j2.b.mass)

####################################################################
if __name__ == "__main__":
    print ("testing pymunk version " + p.version)
    unittest.main()



import pymunk as p
from pymunk.vec2d import Vec2d
import unittest

class UnitTestArbiter(unittest.TestCase):    
    def testRestitution(self):
        s = p.Space()
        s.gravity = 0,-100
        
        b1 = p.Body(1, 1)
        c1 = p.Circle(b1, 10)
        b1.position = 0, 25
        c1.collision_type = 1
        c1.elasticity = 0.6
        
        b2 = p.Body(body_type = p.Body.STATIC)
        c2 = p.Circle(b2, 10)
        c2.collision_type = 2
        c2.elasticity = 0.3
        
        s.add(b1, c1, b2, c2)
        
        def pre_solve(arb, space, data):
            self.assertEqual(arb.restitution, 0.18)
            arb.restitution = 1
            return True
        
        s.add_collision_handler(1,2).pre_solve = pre_solve
        
        for x in range(10):
            s.step(0.1)
            
        self.assertAlmostEqual(b1.position.y, 22.42170317)
        
    def testFriction(self):
        s = p.Space()
        s.gravity = 0,-100
        
        b1 = p.Body(1, p.inf)
        c1 = p.Circle(b1, 10)
        b1.position = 10, 25
        c1.collision_type = 1
        c1.friction = 0.6
        
        b2 = p.Body(body_type = p.Body.STATIC)
        c2 = p.Circle(b2, 10)
        c2.collision_type = 2
        c2.friction = 0.3
        
        s.add(b1, c1, b2, c2)
        
        def pre_solve(arb, space, data):
            self.assertEqual(arb.friction, 0.18)
            arb.friction = 1
            return True
        
        s.add_collision_handler(1,2).pre_solve = pre_solve
        
        for x in range(10):
            s.step(0.1)
            
        self.assertAlmostEqual(b1.position.x, 10.99450928394)
            
    def testSurfaceVelocity(self):
        s = p.Space()
        s.gravity = 0,-100
        
        b1 = p.Body(1, p.inf)
        c1 = p.Circle(b1, 10)
        b1.position = 10, 25
        c1.collision_type = 1
        c1.surface_velocity = (3,0)
        
        b2 = p.Body(body_type = p.Body.STATIC)
        c2 = p.Circle(b2, 10)
        c2.collision_type = 2
        c2.surface_velocity = (5,0)
        
        s.add(b1, c1, b2, c2)
        
        def pre_solve(arb, space, data):
            self.assertAlmostEqual(arb.surface_velocity.x, 1.38461538462)
            self.assertAlmostEqual(arb.surface_velocity.y, -0.923076923077)
            
            arb.surface_velocity = (10,10)
            #TODO: add assert check that setting surface_velocity has any effect
            return True
        
        s.add_collision_handler(1,2).pre_solve = pre_solve
        
        for x in range(5):
            s.step(0.1)    
        
    def testContactPointSet(self):
        s = p.Space()
        s.gravity = 0,-100
        
        b1 = p.Body(1, 30)
        c1 = p.Circle(b1, 10)
        b1.position = 5, 3
        c1.collision_type = 1
        
        b2 = p.Body(body_type = p.Body.STATIC)
        c2 = p.Circle(b2, 10)
        c2.collision_type = 2
        
        s.add(b1, c1, b2, c2)
        
        def pre_solve(arb, space, data):
            # check inital values
            ps = arb.contact_point_set
            self.assertEqual(len(ps.points), 1)
            self.assertAlmostEqual(ps.normal.x, 0.8574929257)
            self.assertAlmostEqual(ps.normal.y, 0.5144957554)
            p1 = ps.points[0]
            self.assertAlmostEqual(p1.point_a.x, 8.574929257)
            self.assertAlmostEqual(p1.point_a.y, 5.144957554)
            self.assertAlmostEqual(p1.point_b.x, -3.574929257)
            self.assertAlmostEqual(p1.point_b.y, -2.144957554)
            self.assertAlmostEqual(p1.distance, -14.16904810)
            
            # check that they can be changed
            ps.normal = Vec2d(1,0)
            ps.points[0].point_a = Vec2d(9,10)
            ps.points[0].point_b = Vec2d(-2,-3)
            ps.points[0].distance = -10
            
            arb.contact_point_set = ps
            ps2 = arb.contact_point_set
            
            self.assertEqual(ps2.normal, (1,0))
            p1 = ps2.points[0]
            self.assertAlmostEqual(p1.point_a, (9,10))
            self.assertAlmostEqual(p1.point_b, (-2,-3))
            self.assertAlmostEqual(p1.distance, -11)
            
            # check for length of points
            ps2.points = []
            def f():
                arb.contact_point_set = ps2    
            self.assertRaises(Exception, f)
            
            return True
            
        s.add_default_collision_handler().pre_solve = pre_solve
        
        s.step(0.1)
        
    def testImpulse(self):
        s = p.Space()
        s.gravity = 0,-100
        
        b1 = p.Body(1, 30)
        c1 = p.Circle(b1, 10)
        b1.position = 5, 3
        c1.collision_type = 1
        c1.friction = 0.5
        
        b2 = p.Body(body_type = p.Body.STATIC)
        c2 = p.Circle(b2, 10)
        c2.collision_type = 2
        c2.friction = 0.8
        
        s.add(b1, c1, b2, c2)
        
        self.post_solve_done = False
        
        def post_solve(arb, space, data):
            self.assertAlmostEqual(arb.total_impulse.x, 3.3936651583)
            self.assertAlmostEqual(arb.total_impulse.y, 4.3438914027)
            self.post_solve_done = True
            
        s.add_collision_handler(1,2).post_solve = post_solve
        
        s.step(0.1)
            
        self.assertTrue(self.post_solve_done)
    
    def testTotalKE(self):
        s = p.Space()
        s.gravity = 0,-100
        
        b1 = p.Body(1, 30)
        c1 = p.Circle(b1, 10)
        b1.position = 5, 3
        c1.collision_type = 1
        c1.friction = 0.5
        
        b2 = p.Body(body_type = p.Body.STATIC)
        c2 = p.Circle(b2, 10)
        c2.collision_type = 2
        c2.friction = 0.8
        
        s.add(b1, c1, b2, c2)
        
        def post_solve(arb, space, data):
            self.assertAlmostEqual(arb.total_ke, 43.438914027)
            return True
        
        s.add_collision_handler(1,2).post_solve = post_solve
        
        s.step(0.1)
        
    def testIsFirstContact(self):
        s = p.Space()
        s.gravity = 0,-100
        
        b1 = p.Body(1, 30)
        c1 = p.Circle(b1, 10)
        b1.position = 5, 3
        c1.collision_type = 1
        c1.friction = 0.5
        
        b2 = p.Body(body_type = p.Body.STATIC)
        c2 = p.Circle(b2, 10)
        c2.collision_type = 2
        c2.friction = 0.8
        
        s.add(b1, c1, b2, c2)
        
        def pre_solve1(arb, space, data):
            self.assertTrue(arb.is_first_contact)
            return True
        
        s.add_collision_handler(1,2).pre_solve = pre_solve1
        
        s.step(0.1)
        
        def pre_solve2(arb, space, data):
            self.assertFalse(arb.is_first_contact)
            return True
        
        s.add_collision_handler(1,2).pre_solve = pre_solve2
        
        s.step(0.1)

    def testNormal(self):
        s = p.Space()
        s.gravity = 0,-100
        
        b1 = p.Body(1, 30)
        b1.position = 5, 10
        c1 = p.Circle(b1, 10)
        c2 = p.Circle(s.static_body, 10)
        
        s.add(b1, c1, c2)
        
        def pre_solve1(arb, space, data):
            self.assertAlmostEqual(arb.normal.x, 0.44721359)
            self.assertAlmostEqual(arb.normal.y, 0.89442719)
            return True
        
        s.add_default_collision_handler().pre_solve = pre_solve1
        
        s.step(0.1)
        
    def testIsRemoval(self):
        s = p.Space()
        s.gravity = 0,-100
        
        b1 = p.Body(1, 30)
        c1 = p.Circle(b1, 10)
        b1.position = 5, 3
        c1.collision_type = 1
        c1.friction = 0.5
        
        b2 = p.Body(body_type = p.Body.STATIC)
        c2 = p.Circle(b2, 10)
        c2.collision_type = 2
        c2.friction = 0.8
        
        s.add(b1, c1, b2, c2)
        
        self.called1 = False
        def separate1(arb, space, data):
            self.called1 = True
            self.assertFalse(arb.is_removal)
        
        s.add_collision_handler(1,2).separate = separate1
        
        for x in range(10):
            s.step(0.1)
        self.assertTrue(self.called1)
        
        b1.position = 5,3
        s.step(0.1) 
        
        self.called2 = False
        def separate2(arb, space, data):
            self.called2 = True
            self.assertTrue(arb.is_removal)
            return True
       
        s.add_collision_handler(1,2).separate = separate2
        s.remove(b1, c1)
        
        self.assertTrue(self.called2)
        
    def testShapes(self):
        s = p.Space()
        s.gravity = 0,-100
        
        b1 = p.Body(1, 30)
        c1 = p.Circle(b1, 10)
        b1.position = 5, 3
        c1.collision_type = 1
        c1.friction = 0.5
        
        b2 = p.Body(body_type = p.Body.STATIC)
        c2 = p.Circle(b2, 10)
        c2.collision_type = 2
        c2.friction = 0.8
        
        s.add(b1, c1, b2, c2)
        
        self.called = False
        def pre_solve(arb, space, data):
            self.called = True
            self.assertEqual(len(arb.shapes), 2)
            self.assertEqual(arb.shapes[0], c1)
            self.assertEqual(arb.shapes[1], c2)
            return True
        
        s.add_collision_handler(1,2).pre_solve = pre_solve
        
        s.step(0.1)
        self.assertTrue(self.called)
        
            
if __name__ == "__main__":
    print ("testing pymunk version " + p.version)
    unittest.main()        


__docformat__ = "reStructuredText"

from weakref import WeakSet
import copy
    
from . import _chipmunk_cffi
cp = _chipmunk_cffi.lib
ffi = _chipmunk_cffi.ffi    
from .vec2d import Vec2d
from .arbiter import Arbiter
from ._pickle import PickleMixin

class Body(PickleMixin, object):
    """A rigid body

    * Use forces to modify the rigid bodies if possible. This is likely to be
      the most stable.
    * Modifying a body's velocity shouldn't necessarily be avoided, but
      applying large changes can cause strange results in the simulation.
      Experiment freely, but be warned.
    * Don't modify a body's position every step unless you really know what
      you are doing. Otherwise you're likely to get the position/velocity badly
      out of sync.

    A Body can be copied and pickled. Sleeping bodies that are copied will be 
    awake in the fresh copy. When a Body is copied any spaces, shapes or 
    constraints attached to the body will not be copied.
    """

    DYNAMIC = cp.CP_BODY_TYPE_DYNAMIC
    """Dynamic bodies are the default body type.

    They react to collisions,
    are affected by forces and gravity, and have a finite amount of mass.
    These are the type of bodies that you want the physics engine to
    simulate for you. Dynamic bodies interact with all types of bodies
    and can generate collision callbacks.
    """

    KINEMATIC = cp.CP_BODY_TYPE_KINEMATIC
    """Kinematic bodies are bodies that are controlled from your code
    instead of inside the physics engine.

    They arent affected by gravity and they have an infinite amount of mass
    so they don't react to collisions or forces with other bodies. Kinematic
    bodies are controlled by setting their velocity, which will cause them
    to move. Good examples of kinematic bodies might include things like
    moving platforms. Objects that are touching or jointed to a kinematic
    body are never allowed to fall asleep.
    """

    STATIC = cp.CP_BODY_TYPE_STATIC
    """Static bodies are bodies that never (or rarely) move.

    Using static bodies for things like terrain offers a big performance
    boost over other body types- because Chipmunk doesn't need to check for
    collisions between static objects and it never needs to update their
    collision information. Additionally, because static bodies don't
    move, Chipmunk knows it's safe to let objects that are touching or
    jointed to them fall asleep. Generally all of your level geometry
    will be attached to a static body except for things like moving
    platforms or doors. Every space provide a built-in static body for
    your convenience. Static bodies can be moved, but there is a
    performance penalty as the collision information is recalculated.
    There is no penalty for having multiple static bodies, and it can be
    useful for simplifying your code by allowing different parts of your
    static geometry to be initialized or moved separately.
    """

    _pickle_attrs_init = ['mass', 'moment', 'body_type']
    _pickle_attrs_general = ['force', 'angle', 'position', 'center_of_gravity', 
        'velocity',  'angular_velocity', 'torque']
    _pickle_attrs_skip = ['is_sleeping', '_velocity_func', '_position_func']

    def __init__(self, mass=0, moment=0, body_type=DYNAMIC):
        """Create a new Body

        Mass and moment are ignored when body_type is KINEMATIC or STATIC.

        Guessing the mass for a body is usually fine, but guessing a moment
        of inertia can lead to a very poor simulation so it's recommended to
        use Chipmunk's moment calculations to estimate the moment for you.

        There are two ways to set up a dynamic body. The easiest option is to
        create a body with a mass and moment of 0, and set the mass or
        density of each collision shape added to the body. Chipmunk will
        automatically calculate the mass, moment of inertia, and center of
        gravity for you. This is probably preferred in most cases. Note that 
        these will only be correctly calculated after the body and shape are 
        added to a space.

        The other option is to set the mass of the body when it's created,
        and leave the mass of the shapes added to it as 0.0. This approach is
        more flexible, but is not as easy to use. Don't set the mass of both
        the body and the shapes. If you do so, it will recalculate and
        overwrite your custom mass value when the shapes are added to the body.

        Examples of the different ways to set up the mass and moment:

        >>> import pymunk
        >>> radius = 2
        >>> mass = 3
        >>> density = 3
        >>> def print_mass_moment(b): 
        ...    print("mass={:.0f} moment={:.0f}".format(b.mass, b.moment))
        
        >>> # Using Shape.density
        >>> s = pymunk.Space()
        >>> b = pymunk.Body()
        >>> c = pymunk.Circle(b, radius)
        >>> c.density = density
        >>> print_mass_moment(b)
        mass=0 moment=0
        >>> s.add(b, c)
        >>> print_mass_moment(b)
        mass=38 moment=75

        >>> # Using Shape.mass
        >>> b = pymunk.Body()
        >>> c = pymunk.Circle(b, radius)
        >>> c.mass = mass
        >>> print_mass_moment(b)
        mass=0 moment=0
        >>> s.add(b, c)
        >>> print_mass_moment(b)
        mass=3 moment=6

        >>> # Using Body constructor
        >>> moment = pymunk.moment_for_circle(mass, 0, radius)
        >>> b = pymunk.Body()
        >>> c = pymunk.Circle(b, radius)
        >>> c.mass = mass
        >>> print_mass_moment(b)
        mass=0 moment=0
        >>> s.add(b, c)
        >>> print_mass_moment(b)
        mass=3 moment=6

        It becomes even more useful to use the mass or density properties of 
        the shape when you attach multiple shapes to one body, like in this 
        example with density:

        >>> # Using multiple Shape.density
        >>> b = pymunk.Body()
        >>> c1 = pymunk.Circle(b, radius, offset=(10,0))
        >>> c1.density = density
        >>> c2 = pymunk.Circle(b, radius, offset=(0,10))
        >>> c2.density = density
        >>> s.add(b, c1, c2)
        >>> print_mass_moment(b)
        mass=75 moment=3921

        """
        if body_type == Body.DYNAMIC:
            self._body = ffi.gc(cp.cpBodyNew(mass, moment), cp.cpBodyFree)
        elif body_type == Body.KINEMATIC:
            self._body = ffi.gc(cp.cpBodyNewKinematic(), cp.cpBodyFree)
        elif body_type == Body.STATIC:
            self._body = ffi.gc(cp.cpBodyNewStatic(), cp.cpBodyFree)

        self._init()

    def _init(self):
        self._position_func = None # To prevent the gc to collect the callbacks.
        self._velocity_func = None # To prevent the gc to collect the callbacks.
        self._position_func_base = None # For pickle
        self._velocity_func_base = None # For pickle

        self._space = None # Weak ref to the space holding this body (if any)

        self._constraints = WeakSet() # weak refs to any constraints attached
        self._shapes = WeakSet() # weak refs to any shapes attached
            
    def __repr__(self):
        if self.body_type == Body.DYNAMIC:
            return 'Body(%r, %r, Body.DYNAMIC)' % (self.mass, self.moment)
        elif self.body_type == Body.KINEMATIC:
            return 'Body(Body.KINEMATIC)'
        elif self.body_type == Body.STATIC:
            return 'Body(Body.STATIC)'

    @classmethod
    def _init_with_body(cls, _body):
        """Only used internally in pymunk."""
        b = cls.__new__(cls)
        b._body = _body
        b._init()
        return b

    def _set_mass(self, mass):
        cp.cpBodySetMass(self._body, mass)
    def _get_mass(self):
        return cp.cpBodyGetMass(self._body)
    mass = property(_get_mass, _set_mass,
        doc="""Mass of the body.""")

    def _set_moment(self, moment):
        cp.cpBodySetMoment(self._body, moment)
    def _get_moment(self):
        return cp.cpBodyGetMoment(self._body)
    moment = property(_get_moment, _set_moment,
        doc="""Moment of inertia (MoI or sometimes just moment) of the body.

        The moment is like the rotational mass of a body.
        """)

    def _set_position(self, pos):
        cp.cpBodySetPosition(self._body, tuple(pos))
    def _get_position(self):
        p = cp.cpBodyGetPosition(self._body)
        return Vec2d._fromcffi(p)
    position = property(_get_position, _set_position,
        doc="""Position of the body.

        When changing the position you may also want to call
        :py:func:`Space.reindex_shapes_for_body` to update the collision 
        detection information for the attached shapes if plan to make any 
        queries against the space.""")

    def _set_center_of_gravity(self, cog):
        cp.cpBodySetCenterOfGravity(self._body, tuple(cog))
    def _get_center_of_gravity(self):
        return Vec2d._fromcffi(cp.cpBodyGetCenterOfGravity(self._body))
    center_of_gravity = property(_get_center_of_gravity,
        _set_center_of_gravity,
        doc="""Location of the center of gravity in body local coordinates.

        The default value is (0, 0), meaning the center of gravity is the
        same as the position of the body.
        """)

    def _set_velocity(self, vel):
        cp.cpBodySetVelocity(self._body, tuple(vel))
    def _get_velocity(self):
        return Vec2d._fromcffi(cp.cpBodyGetVelocity(self._body))
    velocity = property(_get_velocity, _set_velocity,
        doc="""Linear velocity of the center of gravity of the body.""")

    def _set_force(self, f):
        cp.cpBodySetForce(self._body, tuple(f))
    def _get_force(self):
        return Vec2d._fromcffi(cp.cpBodyGetForce(self._body))
    force = property(_get_force, _set_force,
        doc="""Force applied to the center of gravity of the body.

        This value is reset for every time step. Note that this is not the 
        total of forces acting on the body (such as from collisions), but the 
        force applied manually from the apply force functions.""")

    def _set_angle(self, angle):
        cp.cpBodySetAngle(self._body, angle)
    def _get_angle(self):
        return cp.cpBodyGetAngle(self._body)
    angle = property(_get_angle, _set_angle,
        doc="""Rotation of the body in radians.

        When changing the rotation you may also want to call
        :py:func:`Space.reindex_shapes_for_body` to update the collision 
        detection information for the attached shapes if plan to make any 
        queries against the space. A body rotates around its center of gravity, 
        not its position.

        .. Note::
            If you get small/no changes to the angle when for example a
            ball is "rolling" down a slope it might be because the Circle shape
            attached to the body or the slope shape does not have any friction
            set.""")


    def _set_angular_velocity(self, w):
        cp.cpBodySetAngularVelocity(self._body, w)
    def _get_angular_velocity(self):
        return cp.cpBodyGetAngularVelocity(self._body)
    angular_velocity = property(_get_angular_velocity, _set_angular_velocity,
        doc="""The angular velocity of the body in radians per second.""")

    def _set_torque(self, t):
        cp.cpBodySetTorque(self._body, t)
    def _get_torque(self):
        return cp.cpBodyGetTorque(self._body)
    torque = property(_get_torque, _set_torque,
        doc="""The torque applied to the body.

        This value is reset for every time step.""")

    def _get_rotation_vector(self):
        return Vec2d._fromcffi(cp.cpBodyGetRotation(self._body))
    rotation_vector = property(_get_rotation_vector,
        doc="""The rotation vector for the body.""")

    def _get_space(self):
        if self._space != None:
            return self._space._get_self() #ugly hack because of weakref
        else:
            return None
    space = property(_get_space,
        doc="""Get the :py:class:`Space` that the body has been added to (or 
        None).""")

    def _set_velocity_func(self, func):
        @ffi.callback("cpBodyVelocityFunc")
        def _impl(_, gravity, damping, dt):
            return func(self, Vec2d._fromcffi(gravity), damping, dt)

        self._velocity_func_base = func
        self._velocity_func = _impl
        cp.cpBodySetVelocityUpdateFunc(self._body, _impl)
    velocity_func = property(fset=_set_velocity_func,
        doc="""The velocity callback function. 
        
        The velocity callback function is called each time step, and can be 
        used to set a body's velocity.

            ``func(body : Body, gravity, damping, dt)``

        There are many cases when this can be useful. One example is individual 
        gravity for some bodies, and another is to limit the velocity which is 
        useful to prevent tunneling. This is an example of such a callback:

        >>> import pymunk
        >>> body = pymunk.Body(1,2)
        >>> def limit_velocity(body, gravity, damping, dt):
        ...     max_velocity = 1000
        ...     pymunk.Body.update_velocity(body, gravity, damping, dt)
        ...     l = body.velocity.length
        ...     if l > max_velocity:
        ...         scale = max_velocity / l
        ...         body.velocity = body.velocity * scale
        ...
        >>> body.velocity_func = limit_velocity

        """)

    def _set_position_func(self, func):
        @ffi.callback("cpBodyPositionFunc")
        def _impl(_, dt):
            return func(self, dt)
        
        self._position_func_base = func    
        self._position_func = _impl
        cp.cpBodySetPositionUpdateFunc(self._body, _impl)
    position_func = property(fset=_set_position_func,
        doc="""The position callback function. 
        
        The position callback function is called each time step and can be 
        used to update the body's position.

            ``func(body, dt) -> None``
        """)

    def _get_kinetic_energy(self):
        #todo: use ffi method
        #return cp._cpBodyKineticEnergy(self._body)

        vsq = self.velocity.dot(self.velocity)
        wsq = self.angular_velocity * self.angular_velocity
        return (vsq*self.mass if vsq else 0.) + (wsq*self.moment if wsq else 0.)

    kinetic_energy = property(_get_kinetic_energy,
        doc="""Get the kinetic energy of a body.""")


    @staticmethod
    def update_velocity(body, gravity, damping, dt):
        """Default rigid body velocity integration function.

        Updates the velocity of the body using Euler integration.
        """
        cp.cpBodyUpdateVelocity(body._body, tuple(gravity), damping, dt)

    @staticmethod
    def update_position(body, dt):
        """Default rigid body position integration function.

        Updates the position of the body using Euler integration. Unlike the
        velocity function, it's unlikely you'll want to override this
        function. If you do, make sure you understand it's source code
        (in Chipmunk) as it's an important part of the collision/joint
        correction process.
        """
        cp.cpBodyUpdatePosition(body._body, dt)

    def apply_force_at_world_point(self, force, point):
        """Add the force force to body as if applied from the world point.

        People are sometimes confused by the difference between a force and
        an impulse. An impulse is a very large force applied over a very
        short period of time. Some examples are a ball hitting a wall or
        cannon firing. Chipmunk treats impulses as if they occur
        instantaneously by adding directly to the velocity of an object.
        Both impulses and forces are affected the mass of an object. Doubling
        the mass of the object will halve the effect.
        """
        cp.cpBodyApplyForceAtWorldPoint(self._body, tuple(force), tuple(point))

    def apply_force_at_local_point(self, force, point):
        """Add the local force force to body as if applied from the body
        local point.
        """
        cp.cpBodyApplyForceAtLocalPoint(self._body, tuple(force), tuple(point))

    def apply_impulse_at_world_point(self, impulse, point=(0, 0)):
        """Add the impulse impulse to body as if applied from the world point.
        """
        cp.cpBodyApplyImpulseAtWorldPoint(self._body, tuple(impulse), tuple(point))

    def apply_impulse_at_local_point(self, impulse, point=(0, 0)):
        """Add the local impulse impulse to body as if applied from the body
        local point.
        """
        cp.cpBodyApplyImpulseAtLocalPoint(self._body, tuple(impulse), tuple(point))


    def activate(self):
        """Reset the idle timer on a body.

        If it was sleeping, wake it and any other bodies it was touching.
        """
        cp.cpBodyActivate(self._body)

    def sleep(self):
        """Forces a body to fall asleep immediately even if it's in midair.

        Cannot be called from a callback.
        """
        if self._space == None:
            raise Exception("Body not added to space")
        cp.cpBodySleep(self._body)

    def sleep_with_group(self, body):
        """Force a body to fall asleep immediately along with other bodies
        in a group.

        When objects in Pymunk sleep, they sleep as a group of all objects
        that are touching or jointed together. When an object is woken up,
        all of the objects in its group are woken up. 
        :py:func:`Body.sleep_with_group` allows you group sleeping objects 
        together. It acts identically to :py:func:`Body.sleep` if you pass 
        None as group by starting a new group. If you pass a sleeping body 
        for group, body will be awoken when group is awoken. You can use this 
        to initialize levels and start stacks of objects in a pre-sleeping 
        state.
        """
        if self._space == None:
            raise Exception("Body not added to space")
        cp.cpBodySleepWithGroup(self._body, body._body)

    def _is_sleeping(self):
        return bool(cp.cpBodyIsSleeping(self._body))
    is_sleeping = property(_is_sleeping,
        doc="""Returns true if the body is sleeping.""")


    def _set_type(self, body_type):
        cp.cpBodySetType(self._body, body_type)
    def _get_type(self):
        return cp.cpBodyGetType(self._body)
    body_type = property(_get_type
        , _set_type
        , doc="""The type of a body (:py:const:`Body.DYNAMIC`, 
        :py:const:`Body.KINEMATIC` or :py:const:`Body.STATIC`).

        When changing an body to a dynamic body, the mass and moment of
        inertia are recalculated from the shapes added to the body. Custom
        calculated moments of inertia are not preserved when changing types.
        This function cannot be called directly in a collision callback.
        """)


    def each_arbiter(self, func, *args, **kwargs):
        """Run func on each of the arbiters on this body.

            ``func(arbiter, *args, **kwargs) -> None``

            Callback Parameters
                arbiter : :py:class:`Arbiter`
                    The Arbiter
                args
                    Optional parameters passed to the callback function.
                kwargs
                    Optional keyword parameters passed on to the callback function.

        .. warning::

            Do not hold on to the Arbiter after the callback!
        """
        @ffi.callback("cpBodyArbiterIteratorFunc")
        def cf(_body, _arbiter, _data):
            arbiter = Arbiter(_arbiter, self._space)
            func(arbiter, *args, **kwargs)
            
        data = ffi.new_handle(self)
        cp.cpBodyEachArbiter(self._body, cf, data)

    def _get_constraints(self):
        return set(self._constraints)

    constraints = property(_get_constraints,
        doc="""Get the constraints this body is attached to.

        The body only keeps a weak reference to the constraints and a
        live body wont prevent GC of the attached constraints""")

    def _get_shapes(self):
        return set(self._shapes)

    shapes = property(_get_shapes,
        doc="""Get the shapes attached to this body.

        The body only keeps a weak reference to the shapes and a live
        body wont prevent GC of the attached shapes""")

    def local_to_world(self, v):
        """Convert body local coordinates to world space coordinates

        Many things are defined in coordinates local to a body meaning that
        the (0,0) is at the center of gravity of the body and the axis rotate
        along with the body.

        :param v: Vector in body local coordinates
        """
        return Vec2d._fromcffi(cp.cpBodyLocalToWorld(self._body, tuple(v)))

    def world_to_local(self, v):
        """Convert world space coordinates to body local coordinates

        :param v: Vector in world space coordinates
        """
        return Vec2d._fromcffi(cp.cpBodyWorldToLocal(self._body, tuple(v)))

    def velocity_at_world_point(self, point):
        """Get the absolute velocity of the rigid body at the given world
        point

        It's often useful to know the absolute velocity of a point on the
        surface of a body since the angular velocity affects everything
        except the center of gravity.
        """
        return Vec2d._fromcffi(
            cp.cpBodyGetVelocityAtWorldPoint(self._body, tuple(point)))

    def velocity_at_local_point(self, point):
        """ Get the absolute velocity of the rigid body at the given body
        local point
        """
        return Vec2d._fromcffi(
            cp.cpBodyGetVelocityAtLocalPoint(self._body, tuple(point)))

    def __getstate__(self):
        """Return the state of this object
        
        This method allows the usage of the :mod:`copy` and :mod:`pickle`
        modules with this class.
        """
        d = super(Body, self).__getstate__()

        d['special'].append(('is_sleeping', self.is_sleeping))
        d['special'].append(('_velocity_func', self._velocity_func_base))
        d['special'].append(('_position_func', self._position_func_base))

        return d

    def __setstate__(self, state):
        """Unpack this object from a saved state.

        This method allows the usage of the :mod:`copy` and :mod:`pickle`
        modules with this class.
        """
        super(Body, self).__setstate__(state)
        
        for k,v in state['special']:
            if k == 'is_sleeping' and v:
                pass
            elif k == '_velocity_func' and v != None:
                self.velocity_func = v
            elif k == '_position_func' and v != None:
                self.position_func = v
    
    def copy(self):
        """Create a deep copy of this body."""
        return copy.deepcopy(self)



# ----------------------------------------------------------------------------
# pymunk
# Copyright (c) 2007-2016 Victor Blomqvist
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# ----------------------------------------------------------------------------

"""This submodule contains utility functions, mainly to help with polygon
creation.

.. note:: this submodule is deprectade by the autogeometry submodule. Expect it 
to be removed in the next Pymunk release. 

"""
from __future__ import division
__docformat__ = "reStructuredText"

from math import sqrt
from .vec2d import Vec2d

X, Y = 0, 1

try:
    from functools import partial
except ImportError:
    # Python 2.4 support
    def partial(func, *args, **keywords):
        def newfunc(*fargs, **fkeywords):
            newkeywords = keywords.copy()
            newkeywords.update(fkeywords)
            return func(*(args + fargs), **newkeywords)

        newfunc.func = func
        newfunc.args = args
        newfunc.keywords = keywords
        return newfunc

def is_clockwise(points):
    """
    Check if the points given forms a clockwise polygon

    :return: True if the points forms a clockwise polygon
    """
    a = 0
    i, j = 0, 0
    for i in range(len(points)):
        j = i + 1
        if j == len(points): j = 0
        a += points[i][X]*points[j][Y] - points[i][Y]*points[j][X]
    return a <= 0 #or is it the other way around?

def is_left(p0, p1, p2):
    """Test if p2 is left, on or right of the (infinite) line (p0,p1).

    :return: > 0 for p2 left of the line through p0 and p1
        = 0 for p2 on the line
        < 0 for p2 right of the line
    """
    # cast the answer to an int so it can be used directly from sort()
    # cast is not a good idea.. use something else
    #return int((p1.x - p0.x)*(p2.y-p0.y) - (p2.x-p0.x)*(p1.y-p0.y))
    sorting = (p1[X] - p0[X])*(p2[Y]-p0[Y]) - (p2[X]-p0[X])*(p1[Y]-p0[Y])
    if sorting > 0: return 1
    elif sorting < 0: return -1
    else: return 0

def is_convex(points):
    """Test if a polygon (list of (x,y)) is convex or not

    :return: True if the polygon is convex, False otherwise
    """

    assert len(points) > 2, "need at least 3 points to form a polygon"

    p0 = points[0]
    p1 = points[1]
    p2 = points[2]
    xc, yc = 0, 0
    is_same_winding = is_left(p0, p1, p2)
    for p2 in points[2:] + [p0] + [p1]:
        if is_same_winding != is_left(p0, p1, p2):
            return False
        a = p1[X] - p0[X], p1[Y] - p0[Y] # p1-p0
        b = p2[X] - p1[X], p2[Y] - p1[Y] # p2-p1
        if sign(a[X]) != sign(b[X]): xc +=1
        if sign(a[Y]) != sign(b[Y]): yc +=1
        p0, p1 = p1, p2

    return xc <= 2 and yc <= 2

def sign(x):
    """Sign function.

    :return -1 if x < 0, else return 1
    """
    if x < 0: return -1
    else: return 1

def reduce_poly(points, tolerance=0.5):
    """Remove close points to simplify a polyline
    tolerance is the min distance between two points squared.

    :return: The reduced polygon as a list of (x,y)
    """

    assert len(points) > 0, "reduce_poly can not simplify an empty points list"

    curr_p = points[0]
    reduced_ps = [points[0]]

    for p in points[1:]:
        distance = (curr_p[X] - p[X])**2 + (curr_p[Y] - p[Y])**2
        if distance > tolerance:
            curr_p = p
            reduced_ps.append(p)

    return reduced_ps

def convex_hull(points):
    """Create a convex hull from a list of points.
    This function uses the Graham Scan Algorithm.

    :return: Convex hull as a list of (x,y)
    """

    assert len(points) > 2, "need at least 3 points to form a convex hull"

    ### Find lowest rightmost point
    p0 = points[0]
    for p in points[1:]:
        if p[Y] < p0[Y]:
            p0 = p
        elif p[Y] == p0[Y] and p[X] > p0[X]:
            p0 = p
    points.remove(p0)

    ### Sort the points angularly about p0 as center
    f = partial(is_left, p0)
    points.sort(key = _cmp_to_key(f))
    points.reverse()
    points.insert(0, p0)

    ### Find the hull points
    hull = [p0, points[1]]

    for p in points[2:]:

        pt1 = hull[-1]
        pt2 = hull[-2]
        l = is_left(pt2, pt1, p)
        if l > 0:
            hull.append(p)
        else:
            while l <= 0 and len(hull) > 2:
                hull.pop()
                pt1 = hull[-1]
                pt2 = hull[-2]
                l = is_left(pt2, pt1, p)
            hull.append(p)
    return hull

def calc_center(points):
    """Calculate the center of a polygon

    :return: The center (x,y)
    """

    #ref: http://en.wikipedia.org/wiki/Polygon

    assert len(points) > 0, "need at least 1 points to calculate the center"

    area = calc_area(points)

    p1 = points[0]
    cx = cy = 0
    for p2 in points[1:] + [points[0]]:
        tmp = (p1[X]*p2[Y] - p2[X]*p1[Y])
        cx += (p1[X] + p2[X]) * tmp
        cy += (p1[Y] + p2[Y]) * tmp
        p1 = p2
    c = 1 / (6. * area) * cx, 1 / (6. * area) * cy
    return c

def poly_vectors_around_center(pointlist, points_as_Vec2d=True):
    """Rearranges vectors around the center
    If points_as_Vec2d, then return points are also Vec2d, else pos

    :return: pointlist ([Vec2d/pos, ...])
    """

    poly_points_center = []
    cx, cy = calc_center(pointlist)

    if points_as_Vec2d:
        for p in pointlist:
            x = p[X] - cx
            y = p[Y] - cy
            poly_points_center.append(Vec2d((x, y)))

    else:
        for p in pointlist:
            x = p[X] - cx
            y = cy - p[Y]
            poly_points_center.append((x, y))

    return poly_points_center

def calc_area(points):
    """Calculate the area of a polygon

    :return: Area of polygon
    """

    #ref: http://en.wikipedia.org/wiki/Polygon

    if len(points) < 3: return 0

    p1 = points[0]
    a = 0
    for p2 in points[1:] + [points[0]]:
        a += p1[X] * p2[Y] - p2[X] * p1[Y]
        p1 = p2
    a = 0.5 * a

    return a

def calc_perimeter(points):
    """Calculate the perimeter of a polygon

    :return: Perimeter of polygon
    """

    if len(points) < 2: return 0

    p1 = points[0]
    c = 0
    for p2 in points[1:] + [points[0]]:
        c += sqrt((p2[X] - p1[X])**2 + (p2[Y] - p1[Y])**2)
        p1 = p2
    return c

### "hidden" functions

def _cmp_to_key(mycmp):
    'Convert a cmp= function into a key= function, useful for python 3'
    class K(object):
        def __init__(self, obj, *args):
            self.obj = obj
        def __lt__(self, other):
            return mycmp(self.obj, other.obj) < 0
        def __gt__(self, other):
            return mycmp(self.obj, other.obj) > 0
        def __eq__(self, other):
            return mycmp(self.obj, other.obj) == 0
        def __le__(self, other):
            return mycmp(self.obj, other.obj) <= 0
        def __ge__(self, other):
            return mycmp(self.obj, other.obj) >= 0
        def __ne__(self, other):
            return mycmp(self.obj, other.obj) != 0
    return K

def _is_corner(a, b, c):
    # returns if point b is an outer corner
    return not(is_clockwise([a, b, c]))

def _point_in_triangle(p, a, b, c):
    # measure area of whole triangle
    whole = abs(calc_area([a, b, c]))
    # measure areas of inner triangles formed by p
    parta = abs(calc_area([a, b, p]))
    partb = abs(calc_area([b, c, p]))
    partc = abs(calc_area([c, a, p]))
    # allow for potential rounding error in area calcs
    # (not that i've encountered one yet, but just in case...)
    thresh = 0.0000001
    # return if the sum of the inner areas = the whole area
    return ((parta+partb+partc) < (whole+thresh))

def _get_ear(poly):
    count = len(poly)
    # not even a poly
    if count < 3:
        return [], []
    # only a triangle anyway
    if count == 3:
        return poly, []

    # start checking points
    for i in range(count):
        ia = (i-1) % count
        ib = i
        ic = (i+1) % count
        a = poly[ia]
        b = poly[ib]
        c = poly[ic]
        # is point b an outer corner?
        if _is_corner(a,b,c):
            # are there any other points inside triangle abc?
            valid = True
            for j in range(count):
                if not(j in (ia,ib,ic)):
                    p = poly[j]
                    if _point_in_triangle(p,a,b,c):
                        valid = False
            # if no such point found, abc must be an "ear"
            if valid:
                remaining = []
                for j in range(count):
                    if j != ib:
                        remaining.append(poly[j])
                # return the ear, and what's left of the polygon after the ear is clipped
                return [a,b,c], remaining

    # no ear was found, so something is wrong with the given poly (not anticlockwise? self-intersects?)
    return [], []

def _attempt_reduction(hulla, hullb):
    inter = [vec for vec in hulla if vec in hullb]
    if len(inter) == 2:
        starta = hulla.index(inter[1])
        tempa = hulla[starta:]+hulla[:starta]
        tempa = tempa[1:]
        startb = hullb.index(inter[0])
        tempb = hullb[startb:]+hullb[:startb]
        tempb = tempb[1:]
        reduced = tempa+tempb
        if is_convex(reduced):
            return reduced
    # reduction failed, return None
    return None

def _reduce_hulls(hulls):
    count = len(hulls)
    # 1 or less hulls passed
    if count < 2:
        return hulls, False

    # check all hulls in the list against each other
    for ia in range(count-1):
        for ib in range(ia+1, count):
            # see if hulls can be reduced to one
            reduction = _attempt_reduction(hulls[ia], hulls[ib])
            if reduction != None:
                # they can so return a new list of hulls and a True
                newhulls = [reduction]
                for j in range(count):
                    if not(j in (ia,ib)):
                        newhulls.append(hulls[j])
                return newhulls, True

    # nothing was reduced, send the original hull list back with a False
    return hulls, False

### major functions

def triangulate(poly):
    """Triangulates poly and returns a list of triangles

    :Parameters:
        poly
            list of points that form an anticlockwise polygon
            (self-intersecting polygons won't work, results are undefined)
    """
    triangles = []
    remaining = poly[:]
    # while the poly still needs clipping
    while len(remaining) > 2:
        # rotate the list:
        # this stops the starting point from getting stale which sometimes
        # a "fan" of polys, which often leads to poor convexisation
        remaining = remaining[1:]+remaining[:1]
        # clip the ear, store it
        ear, remaining = _get_ear(remaining)
        if ear != []:
            triangles.append(ear)
    # return stored triangles
    return triangles

def convexise(triangles):
    """Reduces a list of triangles (such as returned by triangulate()) to a
    non-optimum list of convex polygons

    :Parameters:
        triangles
            list of anticlockwise triangles (a list of three points) to reduce
    """
    # fun fact: convexise probably isn't a real word
    hulls = triangles[:]
    reduced = True
    # keep trying to reduce until it won't reduce any more
    while reduced:
        hulls, reduced = _reduce_hulls(hulls)
    # return reduced hull list
    return hulls

__all__ = ["is_clockwise", "reduce_poly", "convex_hull", "calc_area",
        "calc_center", "poly_vectors_around_center", "is_convex",
        "calc_perimeter",
        "triangulate", "convexise"]
